"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.render = render;

var _Component = require("../Component");

var _Node = require("../Node");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var ATTRIBUTES_MAP = {
  className: 'class',
  htmlFor: 'for'
};

function isValue(data) {
  return typeof data === 'string' || typeof data === 'number';
}

function isVoid(data) {
  return data === null || data === void 0 || typeof data === 'boolean';
}

function isNode(data) {
  return data instanceof _Node.Node;
}

function isComponent(data) {
  return data.prototype instanceof _Component.Component;
}

function isNameEvent(name) {
  return /^on[A-Z][A-Za-z]+$/.test(name);
}

function toHTMLElementEventName(name) {
  return name.replace(/^on/, '').toLowerCase();
}

function setupHTMLElement(element, attributes, children) {
  var htmlElement = document.createElement(element);
  Object.keys(attributes).forEach(function (key) {
    var name = ATTRIBUTES_MAP[key] || key;
    var value = attributes[key];

    if (name === 'style') {
      Object.keys(value).forEach(function (styleKey) {
        var styleValue = value[styleKey];
        htmlElement.style[styleKey] = styleValue;
      });
    } else if (name === 'html') {
      htmlElement.innerHTML = value;
    } else if (isNameEvent(name)) {
      var eventName = toHTMLElementEventName(name);
      htmlElement.addEventListener(eventName, value);
    } else {
      htmlElement.setAttribute(name, value);
    }
  });
  children.forEach(function (child) {
    var childChunk;

    if (isNode(child)) {
      childChunk = walk(child);
    } else if (isValue(child)) {
      childChunk = document.createTextNode(child);
    } else if (isVoid(child)) {
      return;
    } else {
      throw new Error('Invalid children.');
    }

    htmlElement.appendChild(childChunk);
  });
  return htmlElement;
}

function walk(node) {
  if (!isNode(node)) {
    throw new Error('Invalid node.');
  }

  var _node$attributes = node.attributes,
      ref = _node$attributes.ref,
      attributes = _objectWithoutProperties(_node$attributes, ["ref"]);

  var isHTMLElement = typeof node.element === 'string';
  var isComposedElement = typeof node.element === 'function';
  var chunk;

  if (isHTMLElement) {
    chunk = setupHTMLElement(node.element, attributes, node.children);
    ref && ref(chunk);
  } else if (isComposedElement) {
    // If children is only one element, we pass it directly to simplify data manipulation.
    var children = node.children.length === 1 ? node.children[0] : node.children;

    var props = _objectSpread({}, node.attributes, {
      children: children
    });

    var nextNode;
    var toReference;

    if (isComponent(node.element)) {
      var instance = new node.element(props);
      nextNode = instance.render();
      toReference = instance;
    } else {
      nextNode = node.element(props);
    }

    chunk = walk(nextNode);
    ref && ref(toReference || chunk);
  } else {
    throw new Error('Invalid element.');
  }

  return chunk;
}

function render(parentNode, parentRoot) {
  var dom = walk(parentNode);
  parentRoot.appendChild(dom);
}